package com.nisum.order.repository;


import com.nisum.order.collection.*;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;

import static org.apache.spark.sql.functions.col;

public class ProductOrderService {



    public Dataset<?> loadCollectionDataset(String dbName,String collectionName,Class clazz)
    {
        return sparkSession()
                .read()
                .format("com.mongodb.spark.sql.DefaultSource")
                .option("spark.mongodb.input.uri","mongodb://localhost:27017/"+dbName+"."+collectionName+"?readPreference=primaryPreferred")
                .schema(Encoders.bean(clazz).schema())
                .load();

    }





    private SparkSession sparkSession()
    {
        return SparkSession
                .builder()
                .appName("Product Order Joiner info API")
                .master("local[1]")
                .getOrCreate();
    }

    public Dataset<Row> joinOrderInfo()
    {

        String sourceDBName="product_order_db";

        Dataset<Order> orderDataset= (Dataset<Order>) loadCollectionDataset(sourceDBName,"order", Order.class);

        Dataset<BuyerInfo> buyerInfoDataset= (Dataset<BuyerInfo>) loadCollectionDataset(sourceDBName,"buyer_info", BuyerInfo.class);

        Dataset<BuyerTaxInfo> buyerTaxInfoDataset= (Dataset<BuyerTaxInfo>) loadCollectionDataset(sourceDBName,"buyer_tax_info", BuyerTaxInfo.class);

        Dataset<ShippingAddress> shippingAddressDataset= (Dataset<ShippingAddress>) loadCollectionDataset(sourceDBName,"shipping_address", ShippingAddress.class);


        Dataset<Row> OrderShippingInfo=orderDataset.alias("order")
                .join(shippingAddressDataset.alias("shipping"),col("order._id").equalTo(col("shipping._id")),"inner")
                .drop(col("shipping._id"));


        Dataset<Row> buyerTaxDS=
                buyerInfoDataset.alias("buyer")
                        .join(buyerTaxInfoDataset.alias("tax")
                                ,col("buyer._id").equalTo(col("tax._id")),"inner")
                        .drop(col("tax._id"));


        Dataset<Row> finalDS=OrderShippingInfo
                .join(buyerTaxDS.alias("bt"),col("order._id").equalTo(col("bt._id")),"inner")
                .filter(col("shipping.addressLine1").contains("Nandanavanam"))
                .drop(col("bt._id"));

        finalDS.show();

        return finalDS;
    }


}
